function [q_collection, Q_collection, p_collection, m_collection, mu_collection] = rkmk(a, b, s, ord, I_inv, q, Q, p, m, mu, mu_bar, h, n_steps, n, M, N)
% Initialize the help variables k
k_Q = zeros(n*N, s);
k_m = zeros(n*N, s);

% Calculate the ord first bernoulli numbers
bern = zeros(1, ord);
for i = 0:ord-1
    bern(i+1) = bernoulli(i);
end
bern(2) = -bern(2);

% Initialize the output varibles, the numerical flow on the Lie group
q_collection = zeros(N*n, n_steps);
Q_collection = zeros(N*n, n*n_steps);
p_collection = zeros(N*n, n_steps);
m_collection = zeros(N*n, n_steps);
mu_collection = zeros(N*n, n_steps);

q_collection(:, 1) = q;
Q_collection(:, 1:n) = Q;
p_collection(:, 1) = p;
m_collection(:, 1) = m;
mu_collection(:, 1) = mu;
Q_temp = zeros(size(Q));
m_temp = zeros(size(m));
mu_temp = zeros(size(mu));
for ii = 2:n_steps
    if mod(ii, 100) == 0
        disp('Have now taken step number: ')
        ii
    end
    % Initialize the help variables sigma
    sigma_q = zeros(n*N, 1);
    sigma_Q = zeros(n*N, 1);
    sigma_p = zeros(n*N, 1);
    sigma_m = zeros(n*N, 1);
    
    % Initialize the help variables u
    u_q = zeros(n*N, s);
    u_Q = zeros(n*N, s);
    u_p = zeros(n*N, s);
    u_m = zeros(n*N, s);
    
    % Initialize the help variables v
    v_q = zeros(n*N, s);
    v_Q = zeros(n*N, s);
    v_p = zeros(n*N, s);
    v_m = zeros(n*N, s);
    
    % Applying the Runge Kutta method in the Lie algebra
    for i = 1:s
        for j = 1:i-1
            u_q(:, i) = u_q(:, i) + h*a(i,j)*v_q(:, j);
            u_Q(:, i) = u_Q(:, i) + h*a(i,j)*k_Q(:, j);
            u_p(:, i) = u_p(:, i) + h*a(i,j)*v_p(:, j);
            u_m(:, i) = u_m(:, i) + h*a(i,j)*k_m(:, j);
        end
        q_temp = q + u_q(:, i);
        p_temp = p + u_p(:, i);
       
        for j = 1:N
            Q_temp((j-1)*n+1:j*n, :) = Q((j-1)*n+1:j*n, :)*exp_G(u_Q((j-1)*n+1:j*n, i), n);
            m_temp((j-1)*n+1:j*n) = co_Ad(exp_G(u_Q((j-1)*n+1:j*n, i), n), m((j-1)*n+1:j*n)) + co_dexp(u_Q((j-1)*n+1:j*n, i), u_m((j-1)*n+1:j*n, i), n); 
            mu_temp((j-1)*n+1:j*n) = Q_temp((j-1)*n+1:j*n, :)*mu_bar;
        end
        
        v_q(:, i) = f_q(p_temp, M, n, N);
        v_Q(:, i) = f_Q_rot(m_temp, I_inv, n, N); 
        v_p(:, i) = f_p(q_temp, mu_temp, n, N);
        v_m(:, i) = f_m(q_temp, Q_temp, mu_bar, mu_temp, n, N);
        
        ad_Q = adjoint_map_Q(u_Q(:, i), v_Q(:, i), ord, n, N);
        ad_m = adjoint_map_m(u_Q(:, i), u_m(:, i), v_Q(:, i), v_m(:, i), ord, n, N); 
                      
        k_Q(:, i) = dexpinv(ad_Q, bern, ord);
        k_m(:, i) = dexpinv(ad_m, bern, ord);
    end
    
    for i = 1:s
        sigma_q = sigma_q + h*b(i)*v_q(:, i);
        sigma_Q = sigma_Q + h*b(i)*k_Q(:, i);
        sigma_p = sigma_p + h*b(i)*v_p(:, i);
        sigma_m = sigma_m + h*b(i)*k_m(:, i);
    end
    
    % Mapping the flow in the Lie algebra to the Lie group
       
    q = q + sigma_q;
    p = p + sigma_p;
    
    for i = 1:N
        Q((i-1)*n+1:i*n, :) = Q((i-1)*n+1:i*n, :)*exp_G(sigma_Q((i-1)*n+1:i*n), n);
        m((i-1)*n+1:i*n) = co_Ad(exp_G(sigma_Q((i-1)*n+1:i*n), n), m((i-1)*n+1:i*n)) + co_dexp(sigma_Q((i-1)*n+1:i*n), sigma_m((i-1)*n+1:i*n), n);
        mu((i-1)*n+1:i*n) = Q((i-1)*n+1:i*n, :)*mu_bar;
    end
    
    % Update the output variables, the numerical flow on the Lie group
    q_collection(:, ii) = q;
    Q_collection(:, (ii-1)*n+1 : ii*n) = Q;
    p_collection(:, ii) = p;
    m_collection(:, ii) = m;   
    mu_collection(:, ii) = mu; 

end
end
