close all
clear

% Third order and s-stage RK
% s = 3; % S-stage of RK
% ord = 3; % Orden of RK
% 
% % Coefficients of RK3
% c = [0, 3/4, 17/24];
% b = [13/51, -2/3, 24/17];
% a = zeros(4);
% a(2,1) = 3/4;
% a(3,1) = 119/216;
% a(3,2) = 17/108;

s = 4; % The s-stage of the Runge-Kutta method
ord = 4; % The order of the Runge-Kutta method

% The coefficients of the Runge-Kutta method
c = [0, 1/2, 1/2, 1];
b = [1/6, 1/3, 1/3, 1/6];
a = zeros(4);
a(2,1) = 1/2;
a(3,2) = 1/2;
a(4,3) = 1;

N = 10; % Number of molecules
T = 1; % Upper time limit
n_steps =  100000; 
h = T/n_steps; % The step size

I = diag([1; 1.88; 2.88]); % The moments of inertia
I_inv = diag([1; 1/1.88; 1/2.88]); % The inverse moments of inertia
n = 3; % The size of the Lie group, R^n times SO(n)
sigma = 1;
eps = 1;
M = ones(n*N, 1); % The mass of each water molecule

%Initializing the variables q, Q, p, m, mu and mu_bar in 3D
% rng(12);
% q = randn(n*N,1);
% q = N*q/max(abs(q));
% Q = zeros(n*N, n);
% p = zeros(n*N, 1); 
% m = randn(n*N,1);
% mu = zeros(n*N, 1);
% mu_bar = [0; 1; 1;]; % A fixed reference vector
% 
% for i = 1:N
%     Q((i-1)*n+1:i*n, :) = orth(randn(n));
%     mu((i-1)*n+1:i*n) = Q((i-1)*n+1:i*n, 1:n)*mu_bar;
% end


%Initializing the variables q, Q, p, m, mu and mu_bar in 2D
rng(12);
q = randn(n*N,1);
q(n:n:end) = 0;
q = N*q/max(abs(q));
Q = zeros(n*N, n);
p = zeros(n*N, 1); %randn(n*N,1);
m = zeros(n*N, 1);
mu = zeros(n*N, 1);
mu_bar = [1; 1; 0;]; % A fixed reference vector

for i = 1:N
    Q_loc = [orth(randn(n-1)) zeros(2,1); 0 0 1];
    Q((i-1)*n+1:i*n, :) = Q_loc;
    mu((i-1)*n+1:i*n) = Q((i-1)*n+1:i*n, 1:n)*mu_bar;
end


[q_collection, Q_collection, p_collection, m_collection, mu_collection] = rkmk(a, b, s, ord, I_inv, q, Q, p, m, mu, mu_bar, h, n_steps, n, M, N);
plot_2D(N, q_collection, mu_collection)
plot_err_hamiltonian(T, N, n, n_steps, h, q_collection, p_collection, m_collection, mu_collection, I_inv, sigma, eps, M)

